# -*- coding: utf-8 -*-
#
#       characters.py
#       Copyright 2009 Espen Rønnevik <brightside@quasinet.org>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

from urllib2 import urlopen
from urllib import urlencode
from xml.parsers.expat import ParserCreate

from time import strptime, time
from calendar import timegm

apiURL = "http://api.eve-online.com/account/Characters.xml.aspx"
timeStr = "%Y-%m-%d %H:%M:%S"

class characterList(object):
    def __init__(self):
        self.apiVersion = None
        self.characters = []
        self.expired = None

    def __handleBeginElement(self, element, attrs):
        if element == 'rowset':
            self.__xmlElement.append(attrs['name'])
        else:
            if element == 'row':
                self.__processRows(self.__xmlElement[-1], attrs)
            else:
                self.__processElements(element, attrs)
            self.__xmlElement.append(element)

    def __processElements(self, element, attrs):
        if element == 'eveapi':
            self.apiVersion = attrs['version']

    def __processRows(self, setname, attrs):
        if setname == 'characters':
            self.characters.append(attrs['characterID'])

    def __handleAllData(self, data):
        characters = data.strip()
        if characters:
            self.__handleCharacterData(characters)

    def __handleCharacterData(self, data):
        currentElement = self.__xmlElement[-1]
        
        if currentElement == 'currentTime':
            self.expired =  timegm(strptime(data, timeStr))
        elif currentElement == 'cachedUntil':
            tdelta = timegm(strptime(data, timeStr)) - self.expired
            self.expired = int(time()) + tdelta
        
    def __handleEndElement(self, element):
        self.__xmlElement.pop()

    def parse(self, userID, apiKey):
        self.__xmlElement = []
        postData = urlencode({'userID': userID, 'apikey': apiKey})
        resource = urlopen(apiURL, postData)

        parser = ParserCreate()
        parser.StartElementHandler = self.__handleBeginElement
        parser.CharacterDataHandler = self.__handleAllData
        parser.EndElementHandler = self.__handleEndElement
        parser.ParseFile(resource)

        del self.__xmlElement
