# -*- coding: utf-8 -*-
#
#       charactersheet.py
#       Copyright 2009 Espen Rønnevik <brightside@quasinet.org>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

from urllib2 import urlopen
from urllib import urlencode
from xml.parsers.expat import ParserCreate

from time import strptime, time
from calendar import timegm

from interface.character.attribute import Attribute

apiURL = "http://api.eve-online.com/char/CharacterSheet.xml.aspx"
timeStr = "%Y-%m-%d %H:%M:%S"

class characterSheet(object):
    def __init__(self):
        self.apiVersion = None
        self.attributes = {
            'intelligence': Attribute(),
            'memory': Attribute(),
            'charisma': Attribute(),
            'perception': Attribute(),
            'willpower': Attribute(),
            }
        self.balance = None
        self.bloodLine = None
        self.expired = None
        self.gender = None
        self.id = None
        self.medicalClone = {}
        self.name = None
        self.race = None

    def __handleBeginElement(self, element, attrs):
        self.__processElements(element, attrs)
        self.__xmlElement.append(element)

    def __processElements(self, element, attrs):
        if element == 'eveapi':
            self.apiVersion = attrs['version']

    def __handleAllData(self, data):
        characters = data.strip()
        if characters:
            self.__handleCharacterData(characters)

    def __handleCharacterData(self, data):
        currentElement = self.__xmlElement[-1]
        previousElement = self.__xmlElement[-2]
        
        if currentElement == 'race':
            self.race = data
        elif currentElement == 'bloodLine':
            self.bloodLine = data
        elif currentElement == 'gender':
            self.gender = data

        ## CORPORATION INFO ##
        ## Can be ignored for the time being ##

        elif currentElement == 'cloneName':
            self.medicalClone['name'] = data
        elif currentElement == 'cloneSkillPoints':
            self.medicalClone['skillpoints'] = data
        elif currentElement == 'balance':
            self.balance = data

        elif currentElement in self.attributes.keys():
            self.attributes[currentElement].basevalue = data

        elif currentElement[:11] == 'augmentator':
            attribtype = currentElement[11:]
            attrib = previousElement[:-5]
            self.attributes[attrib].implantBonus[attribtype.tolower()] = data

        elif currentElement == 'currentTime':
            self.expired =  timegm(strptime(data, timeStr))
        elif currentElement == 'cachedUntil':
            tdelta = timegm(strptime(data, timeStr)) - self.expired
            self.expired = int(time()) + tdelta

    def __handleEndElement(self, element):        
        self.__xmlElement.pop()

    def parse(self, userID, apiKey, characterID):
        self.__xmlElement = []

        postData = urlencode({'userID': userID, 'apiKey': apiKey,
                              'characterID': characterID})
        resource = urlopen(apiURL, postData)

        parser = ParserCreate()
        parser.StartElementHandler = self.__handleBeginElement
        parser.CharacterDataHandler = self.__handleCharacterData
        parser.EndElementHandler = self.__handleEndElement
        parser.ParseFile(resource)

        del self.__xmlElement
